# HSPROC - Run image processing tasks on raw Hectospec data
# July 10, 2008
# By Doug Mink, SAO Telescope Data Center

procedure hsproc (images)

string	images=""	{prompt="Images to process"}

bool	fixpreamp=yes	{prompt="Run preampfix to clean overscan (y or n)"}
string	biassec="[1080:1120,3:4610]"	{prompt="Bias section"}
bool	fixpix=yes	{prompt="Fix bad pixels (y or n)"}
string  gain1 = "!GAIN" {prompt="Gain for amplifier 1"}
string  gain2 = "!GAIN" {prompt="Gain for amplifier 2"}
string  gain3 = "!GAIN" {prompt="Gain for amplifier 3"}
string  gain4 = "!GAIN" {prompt="Gain for amplifier 4"}
bool	merge=yes	{prompt="Merge multi-extension images into one or two (y or n)"}
bool	perchip=no	{prompt="Merge amps to one output file per CCD (y or n)"}
bool	flatten=yes	{prompt="Flat field correct spectra using HFLAT (y or n)"}
file	normfile=""	{prompt="Pixel to pixel normalization file from MAKEFLAT"}
bool	cosmic=yes	{prompt="Remove cosmic rays from a list of images (yes or no)"}
real	threshold=20.0	{prompt="Threshold for cosmic ray difference"}
bool	recompute=no	{prompt="Recompute median if already there (y or n)"}
bool	extract=yes	{prompt="Extract spectra to multispec file (y or n)"}
bool	disperse=yes	{prompt="Disperse and linearize spectra (y or n)"}
bool	combine=yes	{prompt="Combine into a single multispec file (y or n)"}
bool	correct=yes	{prompt="Correct for fiber transmission etc. (y or n)"}
bool	split=yes	{prompt="Split into individual 1-D spectra (y or n)"}
bool	skyproc=yes	{prompt="Remove sky background from spectra (y or n)"}
bool	deltemp=yes	{prompt="Delete intermediate image files (yes or no)"}
bool	saveold=yes	{prompt="Save unprocessed image files (yes or no)"}
bool	verbose=yes	{prompt="Print process progress (yes or no)"}

struct *proclist

begin

file	imin, imout, templist, templist1, imu, nf, dir1d, dirsky
string	ims, imsec, temparg, g1, g2, g3, g4, inroot, outroot
bool	vb, fixpre, dt, sv, flat, rc, disp
real	th
int	nfiles, lroot, lout, ifile

	vb = verbose
	dt = deltemp
	sv = saveold
	g1 = gain1
	g2 = gain2
	g3 = gain3
	g4 = gain4
	nf = normfile
	th = threshold
	rc = recompute
	disp = disperse
	ims = images
	flat = flatten
	fixpre = fixpreamp

	if (ims == "") {
	    print ("HSPROC: Run image processing tasks on Hectospec data")
	    print ("Usage: himproc <input images with 4 extensions>")
	    return
	    }

# Make directory in which to save raw multi-extension files
	if (sv) {
	    if (!access ("Raw"))
		mkdir ("Raw")
	    }

#  Make temporary list of input files
	templist = mktemp ("tempproc") // ".list"
	files (ims, >templist)

# Process list of files
	nfiles = 0
	proclist = templist
        while (fscan (proclist, imin) != EOF) {

# Skip file if it cannot be read
	    smatch (imin, ".fits")
            if (!smatch.found) {
                imin = imin // ".fits"
                }
	    if (!access (imin)) {
		print ("HSPROC: ", imin, " cannot be read")
		next
		}
	    nfiles = nfiles + 1

# Do not process images without extensions
	    imgetp.default = "-1"
	    imsec = imin // "[0]"
	    imgetp (imsec, "i_naxis")
	    if (imgetp.value != "0") {
		print ("HSPROC: ", imin, " is not a multi-extension image")
		print ("NAXIS = ", imgetp.value)
		next
		}

# Save Raw file, if requested
	    if (sv) {
		imu = "Raw/" // imin
		if (access (imu))
		    delete (imu)
		copy (imin, imu)
		}

# Fix preamp output using Brian McLeod's task
	    if (fixpre)
		hpreamp1 (imin,saveold=no,deltemp=dt,verbose=vb)

#  Run CCDPROC only if it has not already been run
	    hproc1a (imin,biassec=biassec,fixpix=fixpix,saveold=sv,verbose=vb)

#  Correct for amplifier gain only if it has not already been done
	    hgain1 (imin,gain1=g1,gain2=g2,gain3=g3,gain4=g4,saveold=sv,deltemp=dt,verbose=vb)

# Merge amplifier image extensions into a single image (or two if perchip=yes)
	    if (merge)
		hmerge1 (imin,perchip=perchip,saveold=sv,deltemp=dt,verbose=vb)

# Flatten image by dividing by flat field file made by MAKEFLAT
	    if (flat)
		hflat1 (imin,normfile=nf,saveold=sv,verbose=vb)
	    }

# Remove cosmic rays, if requested
# To use this option, the files in the list must all have the same configuration
	if (cosmic) {
	    if (nfiles > 1) {
		temparg = "@" // templist
		hcosmic (temparg, threshold=th,recompute=rc,deltemp=dt,saveold=sv,verbose=vb)
		}
	    else if (vb) {
		print ("TPROC: Cannot remove cosmic rays from a single file")
		}
	    }

# Extract 1-D spectra
	if (extract) {
	    proclist = templist
            while (fscan (proclist, imin) != EOF) {
		hextract1 (imin, mapfile="")
		if (disp)
		    hlin (imin, deltemp=dt, saveold=sv, verbose=vb)
		}
	    }


# Make combined output file name
	templist1 = ""
	if (nfiles > 1) {
	    smatch (imin,".")
	    if (smatch.found) {
		lout = smatch.start - 1
		imout = substr (imin, 1, lout) // ".ms.fits"
		templist1 = mktemp ("tempproco") // ".list"
		}
	    else
		imout = imin
	    }
	else
	    imout = imin

# Combine files if desired
	if (combine && templist1 != "") {

# First, convert list of FITS files to list of multispec files
	    ifile = 1
	    proclist = templist
	    while (fscan (proclist, imin) != EOF) {
		imroot (imin)
		inroot = imroot.root
		imin = inroot // ".ms.fits"
		if (ifile == 1)
		    print (imin, >templist1)
		else
		    print (imin, >>templist1)
		ifile = ifile + 1
		}
	    temparg = "@" // templist1
	    hscombine (temparg, imout, saveold=sv, deltemp=dt, verbose=vb)
	    if (dt) {
		delete (templist1)
		}
	    }

# Correct spectra for fiber transmission, sky absorption, and red leak
	if (correct) {
	    htran (imout, saveold=sv, deltemp=dt, verbose=vb)
	    }

# Split multispec spectra into individual 1-D spectra
	if (split) {
	    hsplit (imout, deltemp=dt, verbose=vb)
	    }

	if (skyproc) {
	    imroot (imout)
	    outroot = imroot.root
	    dir1d = "1d." // outroot
	    cd (dir1d)
	    dirsky = "../skysub_" // outroot
	    while (access (dirsky)) {
		dirsky = dirsky // "a"
		}
	    if (vb) {
		print ("HSPROC: Writing final spectra to ",dirsky)
		}
	    hskyproc ("*.fits", destdir=dirsky)
	    }

	if (dt) {
	    delete (templist)
	    }
end

# Mar  5 2008	New task based on HPROC which includes all processing
# Mar  6 2008	Set hskyproc output directory to skysub_<pointing>
# Jul 10 2008	Call HPROC1A which deals with bias and dark images OK
